/*
 * Copyright (C) 2008 Collabora Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Pierre-Luc Beaudoin <pierre-luc@pierlux.com>
 */

#ifndef __EMPATHY_MAP_VIEW_H__
#define __EMPATHY_MAP_VIEW_H__

#include <gtk/gtk.h>

G_BEGIN_DECLS

#define EMPATHY_TYPE_MAP_VIEW         (empathy_map_view_get_type ())
#define EMPATHY_MAP_VIEW(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), EMPATHY_TYPE_MAP_VIEW, EmpathyMapView))
#define EMPATHY_MAP_VIEW_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), EMPATHY_TYPE_MAP_VIEW, EmpathyMapViewClass))
#define EMPATHY_IS_MAP_VIEW(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), EMPATHY_TYPE_MAP_VIEW))
#define EMPATHY_IS_MAP_VIEW_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), EMPATHY_TYPE_MAP_VIEW))
#define EMPATHY_MAP_VIEW_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), EMPATHY_TYPE_MAP_VIEW, EmpathyMapViewClass))

typedef struct _EmpathyMapView EmpathyMapView;
typedef struct _EmpathyMapViewClass EmpathyMapViewClass;
typedef struct _EmpathyMapViewPriv EmpathyMapViewPriv;

struct _EmpathyMapView
{
  GtkWindow parent;
  gpointer priv;
};

struct _EmpathyMapViewClass
{
  GtkWindowClass parent_class;
};

GType empathy_map_view_get_type (void);

GtkWidget *empathy_map_view_show (void);

G_END_DECLS

#endif /* __EMPATHY_MAP_VIEW_H__ */
